
/*
  TXT2CLIP.C
  Copia un file di testo in clipboard
  (c) 2001 Aldo Giove (http://www.l-d-x.com/)
*/

#include <windows.h>
#include <stdio.h>
#include <string.h>

/* -------------------------------------------------------------- */

#define MSG_HEADER "TXT2CLIP.EXE v.1.0\n(c) 2001 Aldo Giove "\
                   "(http://www.l-d-x.com/).\n\n"

#define IDD_DIALOGMSG 1

HINSTANCE hInstance = NULL;

void mostraMsg(const char *msg);

/* -------------------------------------------------------------- */

LRESULT CALLBACK fnDlgMsg(HWND hDlg,
                          UINT message,
                          WPARAM wParam,
                          LPARAM lParam)
{
    HWND hctl;
    RECT rectw;
    int x,
        y,
        cx,
        cy;

    switch (message)
    {
    case WM_INITDIALOG:
        hctl = GetDlgItem(hDlg, 101);
        SetWindowText(hctl, MSG_HEADER);
        hctl = GetDlgItem(hDlg, 102);
        SetWindowText(hctl, (const char *)lParam);

        // Centra la finestra
        GetWindowRect(hDlg, &rectw);
        cx = GetSystemMetrics(SM_CXSCREEN);
        cy = GetSystemMetrics(SM_CYSCREEN);
        x = (cx - (rectw.right - rectw.left)) / 2;
        y = (cy - (rectw.bottom - rectw.top)) / 2;
        MoveWindow(hDlg, x, y,
                   (rectw.right-rectw.left),
                   (rectw.bottom-rectw.top),
                   TRUE);
        return TRUE;

    case WM_COMMAND:
        if (LOWORD(wParam) == IDOK ||
            LOWORD(wParam) == IDCANCEL ||
            LOWORD(wParam) == 110)
        {
            EndDialog(hDlg, LOWORD(wParam));
            return TRUE;
        }
        break;
    }
    return FALSE;
}

/* -------------------------------------------------------------- */

int apriDocumento(const char *nomeDocumento, int nCmdShow)
{
	HWND hwnd = NULL;
	HINSTANCE h;
    h = ShellExecute(hwnd, NULL, nomeDocumento, NULL, NULL, nCmdShow);

	if ((long)h <= 32)
        {
            char s[1500];

/*
            SE_ERR_ASSOCINCOMPLETE The filename association is incomplete or invalid.
            SE_ERR_DDEBUSY The DDE transaction could not be completed because other DDE                            transactions were being processed.
            SE_ERR_DDEFAIL The DDE transaction failed.
            SE_ERR_DDETIMEOUT The DDE transaction could not be completed because the
                              request timed out.
            SE_ERR_NOASSOC There is no application associated with the given filename                            extension.
            SE_ERR_SHARE A sharing violation occurred.

            0 The system is out of memory or resources.
            ERROR_FILE_NOT_FOUND The specified file was not found.
            ERROR_PATH_NOT_FOUND The specified path was not found.
            ERROR_BAD_FORMAT The .EXE file is invalid (non-Win32 .EXE or error in .EXE                              image).
*/

        switch ((long)h)
        {
            case SE_ERR_NOASSOC:
                sprintf(s,
                        "Errore: su questo sistema non esiste alcuna "
                        "applicazione "
                        "in grado di aprire il documento '%s'.\r\n\r\n"
                        "Error: on this system there is no application "
                        "that can open document '%s'.",
                        nomeDocumento, nomeDocumento);
                break;

            case ERROR_FILE_NOT_FOUND:
            case ERROR_PATH_NOT_FOUND:
                sprintf(s, "Errore: Documento '%s' non trovato.\r\n\r\n"
                           "Error: Document '%s' not found.",
                       nomeDocumento, nomeDocumento);
                break;

            default:
                sprintf(s, "Errore %ld nell'apertura del documento '%s'.\r\n\r\n"
                           "Error %ld while opening document '%s'.",
                       (long)h, nomeDocumento,
                       (long)h, nomeDocumento);
        }

        mostraMsg(s);
        return 2;
    }
    return 0;
}

/* -------------------------------------------------------------- */

void mostraMsg(const char *msg)
{
    int hDlg = DialogBoxParam(hInstance,
                              (LPCSTR)IDD_DIALOGMSG,
                              NULL,
                              (DLGPROC)fnDlgMsg,
                              (LPARAM)msg);

    switch (hDlg)
    {
        case IDOK:
            break;
        case 110:
            // Apre il browser web
            apriDocumento("http://www.l-d-x.com/index.html", SW_SHOW);
            break;
        case -1:
            MessageBox(NULL,
                       "Errore nella creazione del dialog.",
                       "AVVIA",
                       MB_OK|MB_ICONHAND);
    }
}

/* -------------------------------------------------------------- */

long filesize(FILE *stream)
{
   long curpos, length;

   curpos = ftell(stream);
   fseek(stream, 0L, SEEK_END);
   length = ftell(stream);
   fseek(stream, curpos, SEEK_SET);
   return length;
}

/* -------------------------------------------------------------- */

int txt2clip(const char *nomeDocumento)
{
    char *msg = NULL;
    HGLOBAL txt = NULL;
    FILE *f = fopen(nomeDocumento, "rb");
    if (f)
    {
        long fsize = filesize(f);
        if (fsize > 0)
        {
            void FAR *lpvBuffer;

            txt = GlobalAlloc(0, fsize+1);
            lpvBuffer = GlobalLock(txt);

            if (lpvBuffer)
            {
                size_t n = (size_t)fsize;
                size_t block = 512;
                char *lpv = (char *)lpvBuffer;

                while (n > 0 && !msg)
                {
                    // Legge il file
                    size_t nr;

                    if (n < block)
                        block = n;
                    nr = fread(lpv, 1, block, f);
                    if (nr != block)
                        msg = "Errore nella lettura del file.";

                    lpv += block;
                    n -= block;
                }
                *lpv = '\0'; // Termina la stringa

                if (!msg)
                {
                    GlobalUnlock(txt);
                    if (OpenClipboard(NULL))
                    {
                        if (EmptyClipboard())
                        {
                            SetClipboardData(CF_TEXT, txt);
                            CloseClipboard();
                            txt = NULL;
                        }
                        else
                            msg = "Errore nello svuotamento della clipboard.";
                    }
                    else
                        msg = "Errore nell'apertura della clipboard.";
                }
            }
            else
                msg = "Errore nell'ottenimento del puntatore globale.";

            GlobalUnlock(txt);
        }
    }
    else
        msg = "Errore nell'apertura del file.\n\r"
              "Error opening the file.";

    // Cleanup
    if (txt)
        GlobalFree(txt);

    if (msg)
    {
        mostraMsg(msg);
        return 1;
    }

    return 0;
}

/* -------------------------------------------------------------- */

int PASCAL WinMain(
                    HINSTANCE hInst, /* handle of current instance	*/
                    HINSTANCE hPrevInst, /* handle of previous instance	*/
                    LPSTR lpszCmdLine,	/* address of command line	*/
                    int nCmdShow	/* show-window type (open/icon)	*/
                  )
{
	char nomeDocumento[1024];
    hInstance = hInst;
	nomeDocumento[0] = '\0';

	strcpy(nomeDocumento, lpszCmdLine);
	if (!nomeDocumento[0])
    {
        mostraMsg("Indicare su linea di comando il nome del file di "
                  "testo da copiare in clipboard.\r\n\r\n"
                  "Please specify on the command-line the name of the "
                  "text file to be copied into clipboard.\r\n\r\n"
                  "Esempio/Example:\r\nTXT2CLIP TEXT.TXT");
		return 1;
    }

    return txt2clip(nomeDocumento);
}
